@@ -1,5 +1,7 @@
 from __future__ import annotations
 
+import os
+import signal
 import timeit
 import subprocess
 from typing import Any
@@ -42,6 +44,9 @@ class Command:
         self.status = None
         self.runtime = None
 
+        self.stdout_raw = None
+        self.stderr_raw = None
+
         self.shell = shell
         self.command = command
 
@@ -107,23 +112,23 @@ class Command:
 
         try:
             process = subprocess.Popen(self.command, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
-                                       cwd=path, env=envi, shell=self.shell)
+                                       cwd=path, env=envi, shell=self.shell, preexec_fn=os.setsid)
 
-            self.stdout, self.stderr = process.communicate(timeout=timeout)
+            self.stdout_raw, self.stderr_raw = process.communicate(timeout=timeout)
             self.status = process.returncode
 
         except subprocess.TimeoutExpired:
-            process.kill()
-            self.stdout, self.stderr = process.communicate()
+            os.killpg(os.getpgid(process.pid), signal.SIGTERM)
+            self.stdout_raw, self.stderr_raw = process.communicate()
             self.status = 99
 
         except subprocess.CalledProcessError as e:
-            self.stdout = e.output
-            self.stderr = e.stderr
+            self.stdout_raw = e.output
+            self.stderr_raw = e.stderr
             self.status = e.returncode
 
-        self.stdout = self.stdout.decode('utf-8')
-        self.stderr = self.stderr.decode('utf-8')
+        self.stdout = self.stdout_raw.decode('utf-8')
+        self.stderr = self.stderr_raw.decode('utf-8')
 
     def copy(self, other: Command) -> None:
         '''
@@ -180,3 +185,23 @@ class Command:
             output += self.stderr
 
         return output
+
+    def get_raw_output(self) -> bytes:
+        '''
+        Returns the merged stdout_raw and stderr_raw outputs.
+
+        Parameters:
+            None
+
+        Returns:
+            output      Returns merged stdout_raw and stderr_raw output
+        '''
+        output = b''
+
+        if self.stdout:
+            output += self.stdout
+
+        if self.stderr:
+            output += self.stderr
+
+        return output
