@@ -15,5 +15,12 @@ VALIDATOR_ERROR = 13
 VALUE_ERROR = 14
 RUNTIME_ERROR = 15
 CONDITION_FORMAT_ERROR = 16
+EXTRACT_EXCEPTION = 17
+EXTRACTOR_ERROR = 18
+DUPLICATE_ID_ERROR = 19
+YAML_SYNTAX_ERROR = 20
+MISSING_RESOURCE = 21
+VERSION_MISMATCH = 22
 
 LAST_ERROR = 0
+VERSION = '1.10.0'
